// "top level" elementy
var SVGDoc;
var SVGRoot;
// objekty představující vrstvy
var dynamicLayer;
var colisionLayer;
// řetězec používaný při vytváření nových elementů
var svgns = 'http://www.w3.org/2000/svg'; 

// proměná představující kruhový objekt
var dynObj;
// element šipky
var arrow;
// čas minulého kroku simulace
var lasttime;


// poloha myši
var mouseX = 0.0;
var mouseY = 0.0;

// spuštěná animace? (simulace probíhá pouze pokud se míček hýbe)
var running = false;


// pozice a velikost poloměr důlku
var holeX;
var holeY;
var holeRadius = 15.0;

//
var counter = 0;
var wictory = false;
var counterText;
var wictoryText;

var basicSimulation = false;

// funkce zavolaná po načtení svg
function onloadFunc(evt){
   var target = evt.target; 
   SVGDoc = target.ownerDocument; 
   SVGRoot = SVGDoc.documentElement;
   // načtení kolizních úseček
   initColisionObjects();
   // načtení kruhového objektu
   initDynamicObjects();
   //setTimeout(gameLoop,50);
   //
   // získání elementu šipky
   arrow = SVGDoc.getElementById("Arrow");
   
   // testovací mód (ne minigolf)
   if(arrow == null){
      basicSimulation = true;
      running = true;
      lasttime = new Date().getTime();
      dynObj.setAcceleration(0.0, 1.0);
      setTimeout(basicLoop, 1000 / 60);
      return;
   }
   
   // načtení "důlku"
   var hole = SVGDoc.getElementById("Hole");
   var segList = hole.pathSegList;
   var pathSeg = segList.getItem(0);
   tmpVec[0] = pathSeg.x;
   tmpVec[1] = pathSeg.y;
   processTransforms(tmpVec, hole);
   holeX = tmpVec[0] - 15.0;
   holeY = tmpVec[1];
   
   // nastavení počátečního stavu šipky
   updateArrow();
   //
   counterText = SVGDoc.getElementById("counterText");
   wictoryText = SVGDoc.getElementById("wictoryText");
   if(wictoryText != null) {
      console.log("wictory");
      wictoryText.setAttribute('visibility', 'hidden');
   }
   updateCounterText();
}

// obnoví text počítadla
function updateCounterText(){
	if(counterText !=null) {
	  var textNode = document.createTextNode("Počet odpalů: " + counter);
	  counterText.removeChild(counterText.firstChild);
	  counterText.appendChild(textNode);
	  //counterText.node.textContent="Počet odpalů: " + counter;
  	}
}

// obnoví výherní text
function updateWictoryText(){
   if(wictoryText != null) {
      if(wictory){
            wictoryText.setAttribute('visibility', 'shown');
      }else{
      	    wictoryText.setAttribute('visibility', 'hidden');     
      }
   }
}

// transformuje šipku před odpalem podle pozice myši
function updateArrow(){
	// původní délka 65 px
 var x = dynObj.posX;
 var y = dynObj.posY;
 
 var lx =  mouseX - x;
 var ly =  mouseY - y;
 var length = Math.min(65.0 * 1.5, Math.sqrt(lx*lx + ly*ly));
 
 var scale = length / 65.0;
 var angle = Math.atan2(lx,-ly) * 180.0 / Math.PI;
	
 var transform1 = SVGRoot.createSVGTransform();
 transform1.setTranslate(x, y);
 
 var transform3 = SVGRoot.createSVGTransform();
 transform3.setScale(1.0, scale);	
 
 var transform2 = SVGRoot.createSVGTransform();
 transform2.setRotate(angle, 0.0, 0.0);	
 
 var baseTransform = arrow.transform.baseVal; 
 baseTransform.clear();
 baseTransform.appendItem(transform1);
 baseTransform.appendItem(transform2);
 baseTransform.appendItem(transform3);
}

// zpravovává pohyb myši
function mouseMoved(evt){
 mouseX = evt.clientX;
 mouseY = evt.clientY;
 if(!running && !wictory){
   updateArrow();
 }
}

// zpravovává stisk tlačítek miši
function mousePressed(evt){
 var button = evt.button;
 if(button == 0){
   if(!running && !wictory && !basicSimulation){
     makeForcePulse();
     ++counter;
     updateCounterText();
     arrow.setAttribute('visibility', 'hidden');
     running = true; 
     lasttime = new Date().getTime();
     setTimeout(gameLoop, 1000 / 60);
    }  
} else if (button == 1){
  restart();
}	
}

// zavoláno při "odpalu"
function makeForcePulse(){
  var maxForcePulse = 20.0;	
	
  var x = dynObj.posX;
  var y = dynObj.posY;
 
  var lx =  mouseX - x;
  var ly =  mouseY - y;
  var length = Math.sqrt(lx * lx + ly * ly);
  
  var nx = lx / length;
  var ny = ly / length;
  
  var forcePulse = Math.min(length, 1.5 * 65.0) * maxForcePulse / (1.5 * 65.0);
  
  forcePulseX = forcePulse * nx;
  forcePulseY = forcePulse * ny;
  
  dynObj.setVelocity(forcePulseX, forcePulseY);  
}

// funkce na načtení míčku (původně byla zvažována možnost více kruhových objektů - proto cyklus)
function initDynamicObjects(){
   initDynamicObjectDef();
   dynamicLayer = SVGDoc.getElementById("objects");
   var dynamicObjects = dynamicLayer.childNodes;
   for(var i = 0; i < dynamicObjects.length; ++i){
      var element = dynamicObjects[i];
      //var element = SVGDoc.getElementById("Ball");
      if(element instanceof SVGPathElement){
        dynObj = new DynamicObject();
        dynObj.initFromElement(element);
        //dynObj.setAcceleration(0.0, 1.0);
        break;
      }    
   }
}

// funkce vytvářející fiktivní tření způsobující zpomolování i bez nárazů do překážek
function applyFriction(frictionCoef){
  //var frictionCoef = 0.01 * 0.2;	
  var velocityX = dynObj.velX;
  var velocityY = dynObj.velY;
        // výpočet vlivu tření
  var velocity = Math.sqrt(velocityX * velocityX + velocityY * velocityY);
  if(frictionCoef < velocity){
   dynObj.velX = velocityX - velocityX / velocity * frictionCoef;
   dynObj.velY = velocityY - velocityY / velocity * frictionCoef;
  } else {
   dynObj.velX = 0.0;
   dynObj.velY = 0.0;
  }
}

function restart(){
   dynObj.velX = 0.0;
   dynObj.velY = 0.0;
   dynObj.setStartingPos();
   dynObj.updateGraphics();
   if(!basicSimulation){
     wictory = false;
     updateWictoryText();
     running = false;
     arrow.setAttribute('visibility', 'shown');
     updateArrow();
     counter = 0;
     updateCounterText();
   }
}

// funkce řešící chování míčku poblíž díry
function processHole(){
  var ballRadius = dynObj.radius;
  var ballX = dynObj.posX;
  var ballY = dynObj.posY;
  var lx = holeX - ballX;
  var ly = holeY - ballY;
  var distance = Math.sqrt(lx * lx + ly * ly);
  if(distance < holeRadius){   
    var inVal = holeRadius - distance;
    applyFriction(inVal / holeRadius * 0.1);
    if(inVal < ballRadius){
    	var nx = lx / distance;
    	var ny = ly / distance;
    	//
    	
    	dynObj.velX += nx * 0.15;
    	dynObj.velY += ny * 0.15;
    	return;
    }
       if(dynObj.velX == 0 && dynObj.velY == 0){
        wictory = true;
        running = false;
        updateWictoryText();
        return;
       }
        //applyFriction(0.2);
   }
   if(dynObj.velX == 0 && dynObj.velY == 0){
       running = false;
       arrow.setAttribute('visibility', 'shown');
       updateArrow();
   }
}

// minigolf - hlavní cyklus (provádí se pokud probíhá simulace = míček nestojí)
function gameLoop(evt){	
  if(running){
    var time = new Date().getTime();
    var dt = (time - lasttime);
    var steps = (dt / 2) | 0;
    for(var i = 0; i < steps; ++i){
      dynObj.simulate(0.1);
      applyFriction(0.01 * 0.2);
      processHole();
    }
    dynObj.updateGraphics();
    lasttime += steps * 2;
    setTimeout(gameLoop, 1000 / 60);
  }
}


// obyčejná fyzika
function basicLoop(evt){	
  if(running){	  
    var time = new Date().getTime();
    var dt = (time - lasttime);
    var steps = (dt / 2) | 0;
    for(var i = 0; i < steps; ++i){
      dynObj.simulate(0.1);
    }
    dynObj.updateGraphics();
    lasttime += steps * 2;
    setTimeout(basicLoop, 1000 / 60);
  }
}

// aplikuje transformace spjaté s objekty při načítání ze svg souboru
function processTransforms(point, obj){
	var transforms =  obj.transform.baseVal;
	var lngth = transforms.numberOfItems;
	for(var i = 0; i < lngth; ++i){
	  multiplyBySVGMat(point, transforms.getItem(i).matrix)
	}
}

// násobení souřadnic transformační maticí
function multiplyBySVGMat(point, SVGMatrix){
   var x = point[0];
   var y = point[1];
   point[0] = SVGMatrix.a * x + SVGMatrix.c * y + SVGMatrix.e;
   point[1] = SVGMatrix.b * x + SVGMatrix.d * y + SVGMatrix.f;
}




