  var width;
  var height;
  var prevMouseX = 0;
  var prevMouseY = 0;
  var mouseHold = false;
  
  var arcBallRadius = 240;
  
  var shadedModel = true;

////////////////////////////////////
// MATRIXES, VECTORS, QUATERNIONS //
////////////////////////////////////

  var transformMatrix = mat4.create();
  var projectionMatrix = mat4.create();
  var normalMatrix = mat3.create();
  //
  var startVec = vec3.create();
  var endVec = vec3.create();
  var tmpRot = quat.create();
  var rotation = quat.create();
  var translation = vec3.create();
  var origin = vec3.create();
  
  function initMatrixes(){
    var width = canvas.width;
    var height = canvas.height;
    var aspect = 1.0;
    mat4.perspective(projectionMatrix, Math.PI / 6.0, aspect, 1.0, 1000.0);
    vec3.set(translation, 0.0, 0.0, -3.0);
    vec3.set(origin, -0.5, -0.5, 0.0);
  }
  
  function updateMatrixes(){
    mat4.fromRotationTranslation(transformMatrix, rotation, translation);
    mat4.multiply(transformMatrix, projectionMatrix, transformMatrix);
    mat4.translate(transformMatrix, transformMatrix, origin);
    updateUniforms();
  }
  
  function updateUniforms(){
    renderingProgram.use();
    renderingProgram.setMatrixes(transformMatrix, normalMatrix);
  }
  
  function setRotation(q, v1, v2){
  	q[0] = v2[1] * v1[2] - v2[2] * v1[1];
  	q[1] = v2[2] * v1[0] - v2[0] * v1[2];
  	q[2] = v2[0] * v1[1] - v2[1] * v1[0];
  	q[3] = 1.0 + v2[0] * v1[0] + v2[1] * v1[1] + v2[2] * v1[2];
  }
  
  function getArcBallVector(v, x, y){
  	var xvec = (x - width / 2) / arcBallRadius;
  	var yvec = (height - y - height / 2) / arcBallRadius;
  	var cmp = xvec * xvec + yvec * yvec;
  	var zvec;
  	if(cmp > 1){
  	  var lngth = Math.sqrt(cmp);
  	  xvec /= lngth;
  	  yvec /= lngth;
  	  zvec = 0.0;
  	} else {
  	  zvec = Math.sqrt(1 - cmp);
  	}
  	vec3.set(v, xvec, yvec, zvec);
  }
  
//////////////////
// INPUT EVENTS //
//////////////////
  
  function initInputEvents(){
    canvas.onmousedown = mousePressed;
    canvas.onmouseup = mouseReleased;
    canvas.onmousemove = mouseMoved;
    document.onkeydown = keyPressed;
  }
  
  
  /////// MOUSE //////////

  function mousePressed(event){
    mouseHold = true;
    prevMouseX = event.clientX;
    prevMouseY = event.clientY;
    getArcBallVector(startVec, prevMouseX, prevMouseY);
  }
  
 function mouseReleased(event){
    mouseHold = false;
    var mouseX = event.clientX;
    var mouseY = event.clientY;
    if(mouseX == prevMouseX && mouseY == prevMouseY){
      return;
    }
    getArcBallVector(endVec, mouseX, mouseY);
    setRotation(tmpRot, endVec, startVec);
    quat.normalize(tmpRot, tmpRot);
    quat.multiply(rotation, tmpRot, rotation);
    quat.normalize(rotation, rotation);
    updateMatrixes();
    prevMouseX = mouseX;
    prevMouseY = mouseY;
    var tmpVec = startVec;
    startVec = endVec;
    endVec = tmpVec;
  }
  
  function mouseMoved(event){
    if(mouseHold){
      var mouseX = event.clientX;
      var mouseY = event.clientY;
      if(mouseX == prevMouseX && mouseY == prevMouseY){
        return;
      }
      getArcBallVector(endVec, mouseX, mouseY);
      setRotation(tmpRot, endVec, startVec);
      quat.normalize(tmpRot, tmpRot);
      quat.multiply(rotation, tmpRot, rotation);
      quat.normalize(rotation, rotation);
      updateMatrixes();
      prevMouseX = mouseX;
      prevMouseY = mouseY;
      var tmpVec = startVec;
      startVec = endVec;
      endVec = tmpVec;
    }  
  }
  
  ////// KEYBOARD ///////////
  
  function keyPressed(event){
    var code = event.keyCode;
    if(code == 71){ // G
      showGrid = !showGrid;
    } else if(code == 83){ // S
      shadedModel = !shadedModel;
    } else if (code == 32){ // space
      simulationEnabled = !simulationEnabled;
    } else if(code == 13){
       flag.initFields();   
    }
  }
