
function debugLog(message){
    if(debugMode){
        console.log(message);
    }
}

function glErrorDebug(){
    var error = gl.getError();
    var msg = "";
    if(useWebGLDebugUtils){
       msg = WebGLDebugUtils.glEnumToString(error);
    }
    shadersDebug("glError: " + error +" ("+msg+")");
}

function debugSectionString(name){
    return "////////" + name + "////////\n\n";
}

// EXTENSIONS CHECK

function checkExtensionsDebug(message){
    if(debugCheckExtensions){
      debugLog(message);
    }
}

function checkExtensionsSectionDebug(){
    checkExtensionsDebug(debugSectionString("EXTENSIONS CHECK"));
}

function checkExtensionsGLErrorDebug(){
    if(debugCheckExtensions){
      glErrorDebug();
    }
}

// SHADERS

function shadersSectionDebug(){
    shadersDebug(debugSectionString("SHADERS"));
}

function shadersDebug(message){
    if(debugShaders){
      debugLog(message);
    }
}

function shadersSourceDebug(source){
    if(debugShadersSource){
      shadersDebug("source:\n\n" + source + "\n");
    } 
}

function shadersInfoLogDebug(shader){
    if(debugShadersInfoLog){
      shadersDebug("InfoLog: " + gl.getShaderInfoLog(shader).trim());
    }
}

function shadersGLErrorDebug(){
    if(debugShaders && debugShadersGLError){   	    
      glErrorDebug();
    }  
}

function shadersCompileStatusDebug(status){
    if(debugShadersCompileStatus){   	    
      shadersDebug("COMPILE_STATUS: " + status + "\n");
    }  
}

function shadersTypeDebug(shader){
    if(debugShadersType){
      var type = gl.getShaderParameter(shader, gl.SHADER_TYPE);
      var typeString = "\n";
      if(type == gl.VERTEX_SHADER){
          typeString = "VERTEX_SHADER" + typeString;
      } else if(type == gl.FRAGMENT_SHADER){
          typeString = "FRAGMENT_SHADER" + typeString;
      }
      shadersDebug("SHADER_TYPE: " + typeString);
    }  
}

// PROGRAMS

function programsSectionDebug(){
    checkExtensionsDebug(debugSectionString("PROGRAMS"));
}

function programsDebug(message){
    if(debugPrograms){
      debugLog(message);
    }
}

function programsGLErrorDebug(){
    if(debugPrograms && debugProgramsGLError){   	    
      glErrorDebug();
    }  
}

function programLinkStatusDebug(status, program){
    if(debugProgramsLinkStatus){   	    
      shadersDebug("LINK_STATUS: " + status + "\n");
      if(debugProgramsLinkInfoLog){
        programsDebug(" InfoLog: " + gl.getProgramInfoLog(program).trim() + "\n");
      }
    }  
}

function programValidateStatusDebug(program){
    if(debugProgramsValidateStatus){
      gl.validateProgram(program);
      programsDebug("VALIDATE_STATUS: " + gl.getProgramParameter(program, gl.VALIDATE_STATUS));
      if(debugProgramsValidateInfoLog){
        programsDebug(" InfoLog: " + gl.getProgramInfoLog(program).trim() + "\n");
      }
    }  
}
