  //////////
  // FLAG //
  //////////
  
  var flagLog2Size = cfgLog2Size;
  
   function Flag(log2size){
	var size = 1 << log2size;
	this.size = size; // rozměr v hmotných bodech	
	this.spacing = 1.0 / (size - 1.0); // vzdálenost mezi jednotlivými hm. body
	this.textureSpacing = 1.0 / size; // vzdálenost v texturových součadnicích mezi hm body
	console.log("spacing: " + this.spacing);
	if(cfgIntegrator == "Euler"){
	  this.integrator = new Euler();
	} else if(cfgIntegrator == "NSV"){
	  this.integrator = new NSV();
	} else if (cfgIntegrator == "RK4"){
	  this.integrator = new RK4();
	} else if(cfgIntegrator == "SineWave"){
	  this.integrator = new SineWave();
	} else {
	 confing.log("UNKNOWN INTEGRATOR");
	}
	this.integrator.init(this);
   }
  
  function prepareFlagDef(){
  	  Flag.prototype.init = FlagInit;
  	  Flag.prototype.initGridVBOs = FlagInitGridVBOs;
  	  Flag.prototype.createFields = FlagCreateFields;
  	  Flag.prototype.initFields = FlagInitFields;
  	  Flag.prototype.compute = FlagCompute;
  	  Flag.prototype.render = FlagRender;
  	  Flag.prototype.sineWaves = FlagSineWaves;
  	  Flag.prototype.applyParallelAcceleration = FlagApplyParallelAcceleration;
  	  Flag.prototype.applyDiagonalAcceleration = FlagApplyDiagonalAcceleration;
  	  Flag.prototype.applyCurlAcceleration = FlagApplyCurlAcceleration;
  	  Flag.prototype.applyWindAcceleration = FlagApplyWindAcceleration;
    	  Flag.prototype.applyGravitationalAcceleration = FlagApplyGravitationalAcceleration;	  
  	  Flag.prototype.computeNormals = FlagComputeNormals;
  	  Flag.prototype.computeAcceleration = FlagComputeAcceleration;
  	  Flag.prototype.initUniforms = FlagInitUniforms;
  	  Flag.prototype.readBackPositions = FlagReadBackPositions;
  	  Flag.prototype.addToFloatArray = FlagAddToFloatArray;
  }  
  
  	function FlagInit(){
	  console.log("Flag");
	  this.initGridVBOs();
	  this.createFields();
	  this.initUniforms();
          this.initFields();      
	}

  	function FlagInitGridVBOs(){
  	    // generate verticles
	    var gridVerticles = generateGrid(this.size);
	    this.gridVBO = gl.createBuffer();
	    gl.bindBuffer(gl.ARRAY_BUFFER, this.gridVBO);
	    gl.bufferData(gl.ARRAY_BUFFER, gridVerticles, gl.STATIC_DRAW);
	    gl.bindBuffer(gl.ARRAY_BUFFER, null);
	    // generate indices
	    var gridIndices = generateGridElements(this.size);
	    this.gridElementsVBO = gl.createBuffer();
	    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.gridElementsVBO);
	    gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, gridIndices, gl.STATIC_DRAW);
	    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
	    //
	    var gridSize = this.size - 1;
	    this.triangleCount = (gridSize * gridSize) << 1;
 	}

	function FlagCreateFields(){
            this.positionField = createFieldRef(this.size);
            this.velocityField = createFieldRef(this.size);
            this.accelerationField = createFieldRef(this.size);
            this.tmpField = createFieldRef(this.size);
            this.normalField = createFieldRef(this.size);
    	  if(useFloat){        
      	    this.windField = createSingleFieldRef(this.size, gl.LINEAR);      
	  } else {
	    this.windField = createFieldRef(this.size);
	    
	    this.fieldBuff = new Uint8Array(this.size * this.size * 4);
	    this.fieldBuffFloat = new Float32Array(this.size * this.size * 3);
          }
	}
	
	function FlagInitFields(){
            console.log("initFields");
            preCompute();
            // compute initial positions
            gl.viewport(0, 0, this.size, this.size);  
            initialPositionsProgram.use();
            initialPositionsProgram.setDstField(this.tmpField);
            var step = this.spacing;
            var offset = 0.5 * step;
            var scale = 1.0 / (1.0 - step);
            initialPositionsProgram.setScale(scale, scale);
            initialPositionsProgram.setOffset(-offset, -offset);
            initialPositionsProgram.execute();
            
            if(!useFloat){
            	   // console.log("componentScale: " + this.componentScale[0] + " " + this.componentScale[1] + " " + this.componentScale[2]);
              this.readBackPositions();
              console.log("px 0,0: " + this.fieldBuffFloat[0]+" "+ this.fieldBuffFloat[1]+" "+ this.fieldBuffFloat[2]);
              var size = (this.size - 1) * 3;
              console.log("px size,0: " + this.fieldBuffFloat[size + 0]+" "+ this.fieldBuffFloat[size + 1]+" "+ this.fieldBuffFloat[size + 2]);
            }
            swapFieldRefs(this.positionField, this.tmpField);

            // clear velocity field
            clearField(this.velocityField);
            
            this.computeNormals(); 
            postCompute();
	}
	
	function FlagInitUniforms(){
	  if(!useFloat){
	    this.positionScale =  vec3.fromValues(3.0, 3.0, 3.0);
	    this.velocityScale = vec3.fromValues(3.0, 3.0, 3.0);
	    this.accelerationScale = vec3.fromValues(3.0, 3.0, 3.0);
	    //
	    this.positionDataScale =  vec3.fromValues(1.0 / 3.0, 1.0 /  3.0, 1.0 /  3.0);
	    this.velocityDataScale = vec3.fromValues(1.0 / 3.0, 1.0 /  3.0, 1.0 /  3.0);
	    this.accelerationDataScale = vec3.fromValues(1.0 / 3.0, 1.0 /  3.0, 1.0 /  3.0);
	    //
	    this.positionOffset = vec3.fromValues(-0.5, -0.5, -0.5);
	    this.velocityOffset  = vec3.fromValues(-0.5, -0.5, -0.5);
	    this.accelerationOffset  = vec3.fromValues(-0.5, -0.5, -0.5);
	    //
	    this.positionDataOffset = vec3.fromValues(0.5, 0.5, 0.5);
	    this.velocityDataOffset  = vec3.fromValues(0.5, 0.5, 0.5);
	    this.accelerationDataOffset  = vec3.fromValues(0.5, 0.5, 0.5);
	    //
	    this.componentScale = vec3.fromValues((31.0 * 64.0 * 32.0) / 65535.0, (63.0 * 32.0) / 65535.0, 31.0 / 65535.0);
	    //
	    initialPositionsProgram.setDataOffsetAndScale(this.positionDataOffset, this.positionDataScale);
	    //
	    normalsProgram.use();
	    normalsProgram.setPositionFieldOffset(this.positionOffset);
	    normalsProgram.setPositionFieldScale(this.positionScale);
	    //
	    renderingProgram.use();
	    renderingProgram.setPositionFieldOffset(this.positionOffset);
	    renderingProgram.setPositionFieldScale(this.positionScale);
	    //
	  }
	    //

	    //initialPositionsProgram.se
	}
	
	function FlagReadBackPositions(){
		
          var field = this.tmpField;
	  var length = this.size * this.size;
	  var floatBuffer = this.fieldBuffFloat;
	  for(var i = 0; i < this.size * this.size * 3; ++i){
	  	  floatBuffer[i] = 0.0;
	  }
	  
	  gl.bindFramebuffer(gl.FRAMEBUFFER, FBO);
	  gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture1, 0);
	  if (gl.checkFramebufferStatus(gl.FRAMEBUFFER) == gl.FRAMEBUFFER_COMPLETE) {
	    gl.readPixels(0, 0, this.size, this.size, gl.RGBA, gl.UNSIGNED_BYTE, this.fieldBuff);
	    console.log("complete " + this.fieldBuff[0] + " " + this.fieldBuff[1] + " "+ this.fieldBuff[2]);
	    this.addToFloatArray(0, 1, 2);
	  }
	  gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture2, 0);
	  if (gl.checkFramebufferStatus(gl.FRAMEBUFFER) == gl.FRAMEBUFFER_COMPLETE) {
	    gl.readPixels(0, 0, this.size, this.size, gl.RGBA, gl.UNSIGNED_BYTE, this.fieldBuff);
	    console.log("complete " + this.fieldBuff[0] + " " + this.fieldBuff[1] + " "+ this.fieldBuff[2]);
	    this.addToFloatArray(1, 2, 0);
	  }
	  gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture3, 0);
	  if (gl.checkFramebufferStatus(gl.FRAMEBUFFER) == gl.FRAMEBUFFER_COMPLETE) {
	    
	    gl.readPixels(0, 0, this.size, this.size, gl.RGBA, gl.UNSIGNED_BYTE, this.fieldBuff);
	    console.log("complete " + this.fieldBuff[0] + " " + this.fieldBuff[1] + " "+ this.fieldBuff[2]);
	    this.addToFloatArray(2, 0, 1);
	  }
	  		
	  var offset1 = this.positionOffset[0];
	  var offset2 = this.positionOffset[1];
	  var offset3 = this.positionOffset[2];
		
	  var scale1 = this.positionScale[0];
	  var scale2 = this.positionScale[1];
	  var scale3 = this.positionScale[2];
	  
	  var length = this.size * this.size;
	  if(true){
	  for(var i = 0; i < length; ++i){
	    var index = 3 * i;
	    floatBuffer[index] = (floatBuffer[index] + offset1) * scale1;
	    floatBuffer[index + 1] = (floatBuffer[index + 1] + offset2) * scale2;
	    floatBuffer[index + 2] = (floatBuffer[index + 2] + offset3) * scale3;
	  }
	  }

	  	     //console.log("complete" + floatBuffer[0] + " " + floatBuffer[1] + " "+ floatBuffer[2]);
	}
	
	function FlagAddToFloatArray(componentIndex1, componentIndex2, componentIndex3){
		var floatBuffer = this.fieldBuffFloat;
		var byteBuffer = this.fieldBuff;
		var length = this.size * this.size;
		
		var componentScale = this.componentScale;
		var component1Scale = componentScale[componentIndex1] / 255.0;
		var component2Scale = componentScale[componentIndex2] / 255.0;
		var component3Scale = componentScale[componentIndex3] / 255.0;
				
		for(var i = 0; i < length; ++i){
		  var index1 = 4 * i;
		  var index2 = 3 * i;
		  var component1 = index1 + componentIndex1;
		  var component2 = index1 + componentIndex2;
		  var component3 = index1 + componentIndex3;
		  floatBuffer[index2] += byteBuffer[component1] * component1Scale;
		  floatBuffer[index2 + 1] += byteBuffer[component2] * component2Scale;
		  floatBuffer[index2 + 2] += byteBuffer[component3] * component3Scale;
		}	
	}
	

	function FlagRender(){
	  renderingProgram.use();
	  gl.viewport(0, 0, 640, 640);
	  if(vertexTexture){
	    renderingProgram.setPositionField(this.positionField);
	    renderingProgram.setNormalField(this.normalField);
	    //this.setupPosTextures();
	  } else {
	    // textures => VBOs
	    if(useFloat){
	    } else {
	    }	
	  }
		// lightdir
		// transmat
		// normaltexture
		// color
	  gl.bindBuffer(gl.ARRAY_BUFFER, this.gridVBO);
	  gl.vertexAttribPointer(posAttribute, 2, gl.UNSIGNED_SHORT, true, 0, 0);
	  gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
          gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.gridElementsVBO);
          gl.cullFace(gl.FRONT_AND_BACK);
          if(shadedModel){
            renderingProgram.setColor(0.0, 1.0, 0.0);
	    gl.drawElements(gl.TRIANGLES, this.triangleCount * 3, gl.UNSIGNED_SHORT, 0);
          }
	  if(showGrid){
	    renderingProgram.setColor(0.0, 0.0, 1.0);
	    gl.drawElements(gl.LINES,  this.triangleCount * 3, gl.UNSIGNED_SHORT, 0);
	  }
	}
