  var rectVBO;

  function initGlResourceUtil(){
    initRectVBO();
    initFieldRefDef();
  }
  
function createTexture(size, filter, format, type){
    var texture = gl.createTexture();
    gl.bindTexture(gl.TEXTURE_2D, texture);
    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, filter);
    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, filter);
    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
    gl.texImage2D(gl.TEXTURE_2D, 0, format, size, size, 0, format, type, null);
    return texture;
}

  var swapFieldRefs;
  var clearField;
  
  function FieldRef(){
  }
  
  function initFieldRefDef(){
      FieldRef.prototype.setTexture = setTextureFieldRefFloat; 
      if(useFloat){       
        swapFieldRefs = swapFieldRefsFloat;
        clearField = clearFieldFloat;
      } else {
        FieldRef.prototype.setTextures = setTextureFieldsRefRGB565;
        swapFieldRefs = swapFieldRefsRGB565;
        clearField = clearFieldRGB565;
      }
  }
  
  function setTextureFieldRefFloat(texture){
    this.texture = texture;
  }
  
  function setTextureFieldsRefRGB565(texture1, texture2, texture3){
    this.texture1 = texture1;
    this.texture2 = texture2;
    this.texture3 = texture3;
  }

  function swapFieldRefsFloat(fieldRef1, fieldRef2){
     var texture = fieldRef1.texture;
     fieldRef1.texture = fieldRef2.texture;
     fieldRef2.texture = texture;
  }
  
  function swapFieldRefsRGB565(fieldRef1, fieldRef2){
    var texture1 = fieldRef1.texture1;
    var texture2 = fieldRef1.texture2;
    var texture3 = fieldRef1.texture3;
    fieldRef1.texture1 = fieldRef2.texture1;
    fieldRef1.texture2 = fieldRef2.texture2;
    fieldRef1.texture3 = fieldRef2.texture3;
    fieldRef2.texture1 = texture1;
    fieldRef2.texture2 = texture2;
    fieldRef2.texture3 = texture3; 
  }
  
  function clearFieldFloat(field){
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture, 0);
      gl.clear(gl.COLOR_BUFFER_BIT);
  }
  
  function clearFieldRGB565(field){
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture1, 0);
      gl.clear(gl.COLOR_BUFFER_BIT);
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture2, 0);
      gl.clear(gl.COLOR_BUFFER_BIT);
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, field.texture3, 0);
      gl.clear(gl.COLOR_BUFFER_BIT);
  }
  
 function createFieldRef(size){
  var field = new FieldRef();	
  if(useFloat){
    var texture = createTexture(size, gl.NEAREST, gl.RGB, gl.FLOAT);
    field.setTexture(texture);
  } else {
    var field = new FieldRef();
    var texture1 = createTexture(size, gl.NEAREST, gl.RGB, gl.UNSIGNED_SHORT_5_6_5);
    var texture2 = createTexture(size, gl.NEAREST, gl.RGB, gl.UNSIGNED_SHORT_5_6_5);
    var texture3 = createTexture(size, gl.NEAREST, gl.RGB, gl.UNSIGNED_SHORT_5_6_5);
    field.setTextures(texture1, texture2, texture3);
  }
  return field;
 }
 
  function createSingleFieldRef(size, filtering){
    var field = new FieldRef();	
    var texture = useFloat ? createTexture(size, filtering, gl.RGB, gl.FLOAT) : createTexture(size, filtering, gl.RGB, gl.UNSIGNED_SHORT_5_6_5);
    field.setTexture(texture);
    return field;
  }

  function initRectVBO(){
	var rectVerticles = new Int8Array([0, 0, 0, 1, 1, 0, 1, 1]);
	rectVBO = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, rectVBO);
	gl.bufferData(gl.ARRAY_BUFFER, rectVerticles, gl.STATIC_DRAW);
	gl.bindBuffer(gl.ARRAY_BUFFER, null);
  }
  
function generateGrid(size){
    var verticles = new Uint16Array(size * size << 1);
    var step = 1.0 / size;
    var start = step * 0.5;
    var range = 1.0 - step;
    var gridsize = size - 1;
    for(var i = 0; i < size; ++i){
      for(var j = 0; j < size; ++j){
   	   var pos = i * size + j << 1;
   	   verticles[pos] = (start + range * j / gridsize) * 65535; // x
   	   verticles[pos + 1] = (start + range * i / gridsize) * 65535;	// y
      }
    }
    return verticles;
}

function generateGridElements(size){
	var gridsize = size - 1;
 	var gridTriangles = (gridsize * gridsize) << 1;
 	var indices = new Uint16Array(3 * gridTriangles);
 	for(var i = 0; i < gridsize; ++i){
   	  for(var j = 0; j < gridsize; ++j){
   	   var pos = (i * gridsize + j) * 6;
   	   var offset1 = i * size + j;
   	   var offset2 = offset1 + size;
   	   indices[pos] = offset1;
   	   indices[pos + 1] = offset1 + 1;
   	   indices[pos + 2] = offset2;
   	   indices[pos + 3] = offset2;
   	   indices[pos + 4] = offset1 + 1;
   	   indices[pos + 5] = offset2 + 1;
   	  }
 	}
	return indices;
}

function generateCylinderVerticles(radius, length, circleSegments, LengthSegments){
   var verticles = new FloatArray(circleSegments * (LengthSegments + 1));
   for(var i = 0; i <= LengthSegments; ++i){
     for(var j = 0; i < circleSegments; ++j){
       var index = 3 * (i * circleSegments + j);
       var angle = Math.PI * j / circleSegments;
       verticles[index] = radius * Math.cos(angle); // x
       verticles[index + 1] = i * length / LengthSegments; // y
       verticles[index + 2] = radius * Math.sin(angle); // z
     }
   }
   return verticles;
}

function generateCylinderIndices(radius, length, circleSegments, LengthSegments){
   var indices = new Uint16Array(circleSegments * (LengthSegments + 1));
   for(var i = 0; i < LengthSegments; ++i){
     for(var j = 0; i < circleSegments; ++j){
   	   var pos = (i * circleSegments + j) * 6;
   	   var offset1 = i * circleSegments;
   	   var offset2 = offset1 + circleSegments;
   	   indices[pos] = offset1;
   	   indices[pos + 1] = offset1 + (j + 1) % circleSegments;
   	   indices[pos + 2] = offset2 + j;
   	   indices[pos + 3] = offset2 + j;
   	   indices[pos + 4] = offset1 + (j + 1) % circleSegments;
   	   indices[pos + 5] = offset2 + (j + 1) % circleSegments;
     }
   }
   return indices;
}
