  //////////////
  // PROGRAMS //
  //////////////
  
  var initialPositionsProgram;
  var sineWaveProgram;
  var resetProgram;
  var interalParallelAccelerationProgram;
  var interalDiagonalAccelerationProgram;
  var exteralAccelerationProgram;
  var textureOffsetProgram;
  var scaleProgram;
  var eulerIntegrationProgram;
  var RK4IntegrationProgram;
  var AddAndMulProgram;
  var renderingProgram;
  var normalsProgram;
  var testRenderingProgram;
    
  // ProgramObject constructor
  function ProgramObject(){
  }
  
  function createProgramObjectDef(){
    ProgramObject.prototype.deleteProgram = deleteProgram;
    ProgramObject.prototype.setDstField = setDstField;
    ProgramObject.prototype.initFieldUniforms = initFieldUniforms;
    ProgramObject.prototype.initPositionFieldUniforms = initPositionFieldUniforms;
    ProgramObject.prototype.initVelocityFieldUniforms = initVelocityFieldUniforms;
    ProgramObject.prototype.initAccelerationFieldUniforms = initAccelerationFieldUniforms;
    ProgramObject.prototype.initWindAccelerationFieldUniforms = initWindAccelerationFieldUniforms;
    ProgramObject.prototype.initNormalUniforms = initNormalUniforms;
    ProgramObject.prototype.use = useProgram;
    ProgramObject.prototype.createComputeProgram = createComputeProgram;
    ProgramObject.prototype.createRenderingProgram = createRenderingProgram;
    if(useFloat){
      ProgramObject.prototype.execute = execFloat;
    } else {
      ProgramObject.prototype.execute = exec;
      ProgramObject.prototype.setDataOffsetAndScale = setDataOffsetAndScale;
    }   
  }
    
  function createProgramObject(initFunc){
    	  var programObject = new ProgramObject();
  	  programObject.init = initFunc;
  	  programObject.init();
  	  //console.log("programInfo log:\n\n" + gl.getProgramInfoLog(programObject.program));
  	  return programObject;
  }
  
  function initPrograms(){
  	  createProgramObjectDef();
  	  
  	  initProgramsUtil();
  	  //createTexture3Def();
  	  
  	  programsSectionDebug();
  	  
  	  initialPositionsProgram = createProgramObject(initInitialPositionsProgram);
  	  initialPositionsProgram.setOffset = setOffsetInitialPositionsProgram;
  	  initialPositionsProgram.setScale = setScaleInitialPositionsProgram;
  	  
  	  sineWaveProgram = createProgramObject(initSineWavePositionsProgram);
  	  sineWaveProgram.setSrcField = setField1;
  	  sineWaveProgram.setTime = setTimeSineWave;
  	  
  	  resetProgram = createProgramObject(initResetProgram);
  	  resetProgram.setValue = setValueResetProgram;
  	  
  	  if(useFloat){
  	    interalParallelAccelerationProgram = createProgramObject(initInteralParallelAccelerationProgram); 
  	    interalParallelAccelerationProgram.setSpacing = setSpacing;
  	    interalParallelAccelerationProgram.setNeutralDistance = setNeutralDistanceInteralParallelAcceleration;
  	    interalParallelAccelerationProgram.setSpringFactor = setSpringFactorInteralParallelAcceleration;
  	    interalParallelAccelerationProgram.setDampingFactor = setDampingFactorInteralParallelAcceleration;
  	    interalParallelAccelerationProgram.setSrcField = setField3Float;
  	    interalParallelAccelerationProgram.setAccelerationField = setField3Float;
  	    interalParallelAccelerationProgram.setPositionField = setField1;
  	    interalParallelAccelerationProgram.setVelocityField = setField2;	  
  	  
  	    interalDiagonalAccelerationProgram = createProgramObject(initInteralDiagonalAccelerationProgram); 
  	    interalDiagonalAccelerationProgram.setSpacing = setSpacing;
  	    interalDiagonalAccelerationProgram.setNeutralDistance = setNeutralDistanceInteralParallelAcceleration;
  	    interalDiagonalAccelerationProgram.setSpringFactor = setSpringFactorInteralParallelAcceleration;
  	    interalDiagonalAccelerationProgram.setDampingFactor = setDampingFactorInteralParallelAcceleration;
  	    interalDiagonalAccelerationProgram.setSrcField = setField3Float;
  	    interalDiagonalAccelerationProgram.setAccelerationField = setField3Float;
  	    interalDiagonalAccelerationProgram.setPositionField = setField1;
  	    interalDiagonalAccelerationProgram.setVelocityField = setField2;
  	  }
  
  	  exteralAccelerationProgram = createProgramObject(initExteralAccelerationProgram); 
  	  exteralAccelerationProgram.setSpacing = setSpacing;
  	  exteralAccelerationProgram.setWindVector = setWindVecExteralAccelerationProgram;
  	  exteralAccelerationProgram.setDragCoeficient = setDragCoeficientExteralAccelerationProgram;
  	  exteralAccelerationProgram.setPositionField = setField1;
  	  exteralAccelerationProgram.setVelocityField = setField2;
  	  
  	  textureOffsetProgram = createProgramObject(initTextureOffsetProgram);
  	  textureOffsetProgram.setAccelerationField = setField1;
  	  textureOffsetProgram.setWindAccelerationField = setField2;
  	  textureOffsetProgram.setSpacing = setSpacing;
  	  textureOffsetProgram.setOffset = setOffsetTextureOffsetProgram;
  	  
  	  scaleProgram = createProgramObject(initScaleProgram);
  	  scaleProgram.setSrcField = setField1;
  	  scaleProgram.setScale = setScaleScaleProgram;
  	  
  	  eulerIntegrationProgram = createProgramObject(initEulerIntegrationProgram);
  	  eulerIntegrationProgram.setTime = setTimeEulerIntegrationProgram;
  	  eulerIntegrationProgram.setSrcField = setField1;
  	  eulerIntegrationProgram.setDerivativeField = setField2;
  	  if(useFloat){
  	    RK4IntegrationProgram = createProgramObject(initRK4IntegrationProgram);
  	    RK4IntegrationProgram.setTime = setTimeEulerIntegrationProgram;
  	    RK4IntegrationProgram.setSrcField = setField1Float;
  	    RK4IntegrationProgram.setK1Field = setField2Float;
  	    RK4IntegrationProgram.setK2Field = setField3Float;
  	    RK4IntegrationProgram.setK3Field = setField4Float;
  	    RK4IntegrationProgram.setK4Field = setField5Float;
  	  }
  	  
  	  AddAndMulProgram = createProgramObject(initAddAndMulProgram);
  	  AddAndMulProgram.setField1 = setField1;
  	  AddAndMulProgram.setField2 = setField2;
  	  AddAndMulProgram.setScale = setScaleScaleProgram;
  	  
  	  normalsProgram = createProgramObject(initNormalProgram);
  	  normalsProgram.setSpacing = setSpacing;
  	  normalsProgram.setPositionField = setField1;
  	  if(!useFloat){
  	    normalsProgram.execute = execFloat;
  	  }

  	  renderingProgram = createProgramObject(initRenderingProgram);
  	  renderingProgram.setMatrixes = setMatrixesRenderingProgram;
  	  renderingProgram.setColor = setColorRenderingProgram;
  	  renderingProgram.setPositionField = setField1;
  	  renderingProgram.setNormalField = setNormalField;
  	  //testRenderingProgram = createProgramObject(initTestRenderingProgram);	  
  }
  
  // INITIAL POSITIONS //
  
  function initInitialPositionsProgram(){
    programsDebug("initialPositionsProgram");
    this.createComputeProgram(initialFS);
    gl.useProgram(this.program);
    // uniforms
    this.offsetUniform = gl.getUniformLocation(this.program, "offset");
    this.scaleUniform = gl.getUniformLocation(this.program, "scale");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setOffsetInitialPositionsProgram(x, y){
    gl.uniform2f(this.offsetUniform, x, y);	  
  }
  
  function setScaleInitialPositionsProgram(x, y){
    gl.uniform2f(this.scaleUniform, x, y);
  }
  
  // SINE WAVE //
  
  function initSineWavePositionsProgram(){
    programsDebug("sineWaveProgram");
    this.createComputeProgram(sineWaveFS);
    gl.useProgram(this.program);
    this.initPositionFieldUniforms(0);
    // uniforms
    this.timeUniform = gl.getUniformLocation(this.program, "time");
    gl.uniform1f(this.timeUniform, 0.0);
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setTimeSineWave(time){
    gl.uniform1f(this.timeUniform, time);
  }

  function initResetProgram(){
    programsDebug("resetProgram");
    this.createComputeProgram(resetFS);
    gl.useProgram(this.program);
    this.valueUniform = gl.getUniformLocation(this.program, "value");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setValueResetProgram(x, y, z){
    gl.uniform3f(this.valueUniform, x, y, z);
  }
  
  // INTERAL PARALLEL ACCELERATIONS
  
  function initInteralParallelAccelerationProgram(){
    programsDebug("interalParallelAccelerationProgram");
    this.createComputeProgram(interalParallelAccelerationFS);
    gl.useProgram(this.program);  
    this.initPositionFieldUniforms(0);
    this.initVelocityFieldUniforms(1);
    this.initAccelerationFieldUniforms(2);
    this.spacingUniform = gl.getUniformLocation(this.program, "spacing");
    this.neutralDistanceUniform = gl.getUniformLocation(this.program, "l0");
    this.springFactorUniform = gl.getUniformLocation(this.program, "kp");
    this.damperFactorUniform = gl.getUniformLocation(this.program, "kt");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setSpringFactorInteralParallelAcceleration(factor){
    gl.uniform1f(this.springFactorUniform, factor);
  }
  
  function setDampingFactorInteralParallelAcceleration(factor){
    gl.uniform1f(this.damperFactorUniform, factor);
  }
  
  function setNeutralDistanceInteralParallelAcceleration(distance){
    gl.uniform1f(this.neutralDistanceUniform, distance);
  }
  
  // INTERAL DIAGONAL ACCELERATIONS
  
  function initInteralDiagonalAccelerationProgram(){
    programsDebug("interalDiagonalAccelerationProgram");
    this.createComputeProgram(interalDiagonalAccelerationFS);
    gl.useProgram(this.program);
    this.initPositionFieldUniforms(0);
    this.initVelocityFieldUniforms(1);
    this.initAccelerationFieldUniforms(2);
    this.spacingUniform = gl.getUniformLocation(this.program, "spacing");
    this.neutralDistanceUniform = gl.getUniformLocation(this.program, "l0");
    this.springFactorUniform = gl.getUniformLocation(this.program, "kp");
    //console.log("kp " + this.springFactorUniform+"\n");
    this.damperFactorUniform = gl.getUniformLocation(this.program, "kt");
   // console.log("kt " + this.damperFactorUniform+"\n");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  // EXTERAL ACCELERATIONS
  
  function initExteralAccelerationProgram(){
    programsDebug("exteralAccelerationProgram");
    this.createComputeProgram(exteralAccelerationFS);
    gl.useProgram(this.program);
    this.initPositionFieldUniforms(0);
    this.initVelocityFieldUniforms(1);
    this.spacingUniform = gl.getUniformLocation(this.program, "spacing");
    this.windVecUniform = gl.getUniformLocation(this.program, "windVec");
    this.dragCoeficientUniform = gl.getUniformLocation(this.program, "cx");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setWindVecExteralAccelerationProgram(x, y, z){
    gl.uniform3f(this.windVecUniform, x, y, z);
  }
  
  function setDragCoeficientExteralAccelerationProgram(cx){
    gl.uniform1f(this.dragCoeficientUniform, cx);
  }
  
  // TEXTURE OFFSET
  
  function initTextureOffsetProgram(){
    programsDebug("textureOffsetProgram");
    this.createComputeProgram(textureOffsetFS);
    gl.useProgram(this.program);
    this.initAccelerationFieldUniforms(0);
    this.initWindAccelerationFieldUniforms(1);
    this.offsetUniform = gl.getUniformLocation(this.program, "offset");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setOffsetTextureOffsetProgram(x, y){
    gl.uniform2f(this.offsetUniform, x, y);
  }
  
  // SCALE
  
  function initScaleProgram(){
    programsDebug("scaleProgram");
    this.createComputeProgram(scaleFS);
    gl.useProgram(this.program);
    this.initPositionFieldUniforms(0);
    this.scaleUniform = gl.getUniformLocation(this.program, "scale");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setScaleScaleProgram(x, y, z){
    gl.uniform3f(this.scaleUniform, x, y, z);
  }
  
  // EULER INTEGRATION
  
  function initEulerIntegrationProgram(){
    programsDebug("eulerIntegrationProgram");
    this.createComputeProgram(eulerIntegrationFS);
    gl.useProgram(this.program);
    this.initPositionFieldUniforms(0);
    this.initVelocityFieldUniforms(1);
    this.timeUniform = gl.getUniformLocation(this.program, "time");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setTimeEulerIntegrationProgram(time){
    gl.uniform1f(this.timeUniform, time);  
  }
  
  // RK4 INTEGRATION
  function initRK4IntegrationProgram(){
    programsDebug("RK4IntegrationProgram");
    this.createComputeProgram(RK4IntegrationFS);
    gl.useProgram(this.program);
    this.initFieldUniforms("src", 0);
    this.initFieldUniforms("k1", 1);
    this.initFieldUniforms("k2", 2);
    this.initFieldUniforms("k3", 3);
    this.initFieldUniforms("k4", 4);
    this.timeUniform = gl.getUniformLocation(this.program, "time");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  // MUL AND ADD
    function initAddAndMulProgram(){
      programsDebug("AddAndMulProgram");
      this.createComputeProgram(AddAndMulFS);
      gl.useProgram(this.program);
      this.initFieldUniforms("k1", 0);
      this.initFieldUniforms("k2", 1);
      this.scaleUniform = gl.getUniformLocation(this.program, "scale");
      programsGLErrorDebug();
      programsDebug("\n");
  }
  
  // NORMALS //
  
  function initNormalProgram(){
    programsDebug("normalProgram");
    this.createComputeProgram(normalFS);
    gl.useProgram(this.program);
    // uniforms
    this.initPositionFieldUniforms(0);
    this.spacingUniform = gl.getUniformLocation(this.program, "spacing");
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  // RENDERING //
   
  function initRenderingProgram(){
    programsDebug("renderingProgram");
    this.createRenderingProgram(renderingFS);
    gl.useProgram(this.program);
    // uniforms
    this.initPositionFieldUniforms(0);
    this.initNormalUniforms();
    this.lightVecUniform = gl.getUniformLocation(this.program, "lightVec");
    this.colorUniform = gl.getUniformLocation(this.program, "color");
    gl.uniform3f(this.colorUniform, 0.0, 1.0, 0.0);
    gl.uniform3f(this.lightVecUniform, 0.0, 0.0, 1.0);
    this.transMatUniform = gl.getUniformLocation(this.program, "transformationMatrix");
    this.normalMatUniform = gl.getUniformLocation(this.program, "normalMatrix");
    //console.log("transformMatrix: " + transformMatrix);
    gl.uniformMatrix4fv(this.transMatUniform, false, transformMatrix);
    gl.uniformMatrix3fv(this.normalMatUniform, false, normalMatrix);
    programsGLErrorDebug();
    programsDebug("\n");
  }
  
  function setMatrixesRenderingProgram(transformMat, normalMat){
    gl.uniformMatrix4fv(this.transMatUniform, false, transformMat);
    gl.uniformMatrix3fv(this.normalMatUniform, false, normalMat);
  }
  
  function setColorRenderingProgram(r, g, b){
    gl.uniform3f(this.colorUniform, r, g, b);
  }
  
  // TEST RENDERING //
  
  function initTestRenderingProgram(){
    programsDebug("testRenderingProgram");
    this.createComputeProgram(testRenderingFS);
    gl.useProgram(testRenderingProg);
    // uniforms
    textureUniformTestRendering = gl.getUniformLocation(testRenderingProg, "texture");
    programsGLErrorDebug();
    programsDebug("\n");
  }
