///////////////////
// PROGRAMS UTIL //
///////////////////

  function createComputeProgram(FS){
    programsDebug("type: compute program");
    this.program = gl.createProgram();
    gl.attachShader(this.program, computeVS);
    gl.attachShader(this.program, FS);
    gl.bindAttribLocation(this.program, posAttribute, "pos"); 
    gl.linkProgram(this.program);
    var status = gl.getProgramParameter(this.program, gl.LINK_STATUS);
    programLinkStatusDebug(status, this.program)
    if(!status){
       throw "Compilation of program failed"   
    }
    programValidateStatusDebug(this.program);
    if(!useFloat){
        this.dataOffsetUniform = gl.getUniformLocation(this.program, "dataOffset");
        this.dataScaleUniform = gl.getUniformLocation(this.program, "dataScale");
        this.dataComponentsIndexUniform = gl.getUniformLocation(this.program, "dataComponentsIndex");
    } 
  }

  function createRenderingProgram(FS){
    programsDebug("type: rendering program");
    this.program = gl.createProgram();
    gl.attachShader(this.program, renderingVS);
    gl.attachShader(this.program, FS);    
    if(vertexTexture){
      gl.bindAttribLocation(this.program, posAttribute, "inTexCoor");
    } else {
    	gl.bindAttribLocation(this.program, posAttribute, "pos");
	gl.bindAttribLocation(this.program, normalAttribute, "normal");
    }
    gl.linkProgram(this.program);
    var status = gl.getProgramParameter(this.program, gl.LINK_STATUS);
    programLinkStatusDebug(status, this.program);
    if(!status){
       throw "Compilation of program failed"   
    }
    programValidateStatusDebug(this.program);
  }
   
  // UNIFORM HELPER FUNCTIONS
    
  function initFieldUniforms(name, offset){
    if(useFloat){
	var fieldUniform = gl.getUniformLocation(this.program, name + "Field");
	gl.uniform1i(fieldUniform, offset);
    } else { 
      var field1Uniform = gl.getUniformLocation(this.program, name + "Field1");
      var field2Uniform = gl.getUniformLocation(this.program, name + "Field2");
      var field3Uniform = gl.getUniformLocation(this.program, name + "Field3");
      var offset1 = 3 * offset;
      gl.uniform1i(field1Uniform, offset1);
      gl.uniform1i(field2Uniform, offset1 + 1);
      gl.uniform1i(field3Uniform, offset1 + 2);
    }
  }
  
    
  function initPositionFieldUniforms(offset){
      this.initFieldUniforms("position", offset);
      if(!useFloat){
          gl.useProgram(this.program);
          this.positionOffsetUniform = gl.getUniformLocation(this.program, "positionOffset");
          this.positionScaleUniform = gl.getUniformLocation(this.program, "positionScale");
          gl.uniform3f(this.positionOffsetUniform, 0.5, 0.5, 0.5);
          gl.uniform3f(this.positionScaleUniform, 2.5, 2.5, 2.5);
          this.setPositionFieldOffset = setPositionFieldOffset;
          this.setPositionFieldScale = setPositionFieldScale;
      }
  }
  
   function setPositionFieldOffset(offset){
       gl.uniform3f(this.positionOffsetUniform, offset[0], offset[1], offset[2]);
   }
  
   function setPositionFieldScale(scale){
       gl.uniform3f(this.positionScaleUniform, scale[0], scale[1], scale[2]);
   }
  
  function initVelocityFieldUniforms(offset){
      this.initFieldUniforms("velocity", offset);
      if(!useFloat){
          gl.useProgram(this.program);
          this.velocityOffsetUniform = gl.getUniformLocation(this.program, "velocityOffset");
          this.velocityScaleUniform = gl.getUniformLocation(this.program, "velocityScale");
          gl.uniform3f(this.velocityOffsetUniform, 0.5, 0.5, 0.5);
          gl.uniform3f(this.velocityScaleUniform, 2.5, 2.5, 2.5);
          this.setVelocityFieldOffset = setVelocityFieldOffset;
          this.setVelocityFieldScale = setVelocityFieldScale;
      }
  }
  
   function setVelocityFieldOffset(offset){
       gl.setUniform3f(this.velocityOffsetUniform, offset);
   }
  
   function setVelocityFieldScale(scale){
       gl.setUniform3f(this.velocityScaleUniform, scale);
   }
  
  function initAccelerationFieldUniforms(offset){
      this.initFieldUniforms("acceleration", offset);
      if(!useFloat){
          gl.useProgram(this.program);
          this.accelerationOffsetUniform = gl.getUniformLocation(this.program, "accelerationOffset");
          this.accelerationScaleUniform = gl.getUniformLocation(this.program, "accelerationScale");
          gl.uniform3f(this.accelerationOffsetUniform, 0.5, 0.5, 0.5);
          gl.uniform3f(this.accelerationScaleUniform, 25, 25, 25);
          this.setAccelerationFieldOffset = setAccelerationFieldOffset;
          this.setAccelerationFieldScale = setAccelerationFieldScale;
      }
  }
  
   function setAccelerationFieldOffset(offset){
       gl.setUniform3f(this.accelerationOffsetUniform, offset);
   }
  
   function setAccelerationFieldScale(scale){
       gl.setUniform3f(this.accelerationScaleUniform, scale);
   }
  
     
   function initWindAccelerationFieldUniforms(offset){
      this.initFieldUniforms("windAcceleration", offset);
      if(!useFloat){
          gl.useProgram(this.program);
          this.windAccelerationOffsetUniform = gl.getUniformLocation(this.program, "windAccelerationOffset");
          this.windAccelerationScaleUniform = gl.getUniformLocation(this.program, "windAccelerationScale");
          gl.uniform3f(this.accelerationOffsetUniform, 0.5, 0.5, 0.5);
          gl.uniform3f(this.accelerationScaleUniform, 25, 25, 25);
          this.setWindAccelerationFieldOffset = setWindAccelerationFieldOffset;
          this.setWindAccelerationFieldScale = setWindAccelerationFieldScale;
      }
  }
  
  function setWindAccelerationFieldOffset(offset){
      gl.setUniform3f(this.windAccelerationOffsetUniform, offset);
  }
  
  function setWindAccelerationFieldScale(scale){
      gl.setUniform3f(this.windAccelerationScaleUniform, scale);
  }
  
  function deletePrograms(){
  }
  
  function initNormalUniforms(){
    this.normalUniform = gl.getUniformLocation(this.program, "normalField");
    if(useFloat){
      gl.uniform1i(this.normalUniform, 1);
    } else {
      gl.uniform1i(this.normalUniform, 3);
    }
  }
      
  // OTHER HELPER FUNCTIONS
  
  function setDstField(field){
  	  this.dstField = field;
  }
  
  function execFloat(){
  	gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, this.dstField.texture, 0);
  	executeComputeProgram();
  }
  
  function setDataOffsetAndScale(dataOffset, dataScale){
  	  this.dataOffset = dataOffset;
  	  this.dataScale = dataScale;
  }
  
  function exec(){
  	var field = this.dstField;
  	console.log(field);
  	var offset = this.dataOffset;
   	var scale = this.dataScale;
        console.log(offset);
        console.log(scale);
  	 
   	gl.uniform3f(this.dataOffsetUniform, offset[0], offset[1], offset[2]);
  	gl.uniform3f(this.dataScaleUniform, scale[0], scale[1], scale[2]);
  	//gl.uniform3f(this.dataOffsetUniform, 0.0, 0.0, 0.0);
  	//gl.uniform3f(this.dataScaleUniform, 1.0, 1.0, 1.0);
  		 
  	gl.uniform1i(this.dataComponentsIndexUniform , 0);	  
    	gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, this.dstField.texture1, 0);
  	executeComputeProgram();
  	//gl.readPixels(0, 0, flag.size, flag.size, gl.RGBA, gl.UNSIGNED_BYTE, flag.fieldBuff);
	//console.log("fs " + flag.fieldBuff[0] + " " + flag.fieldBuff[1] + " "+ flag.fieldBuff[2]);
  	
  	gl.uniform1i(this.dataComponentsIndexUniform, 1); 	
  	gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, this.dstField.texture2, 0);
  	executeComputeProgram();
  	//gl.readPixels(0, 0, flag.size, flag.size, gl.RGBA, gl.UNSIGNED_BYTE, flag.fieldBuff);
	//console.log("fs " + flag.fieldBuff[0] + " " + flag.fieldBuff[1] + " "+ flag.fieldBuff[2]);
  	
  	gl.uniform1i(this.dataComponentsIndexUniform, 2);
  	gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, this.dstField.texture3, 0);
  	executeComputeProgram();
  	//gl.readPixels(0, 0, flag.size, flag.size, gl.RGBA, gl.UNSIGNED_BYTE, flag.fieldBuff);
	//console.log("fs " + flag.fieldBuff[0] + " " + flag.fieldBuff[1] + " "+ flag.fieldBuff[2]);
  }
  
  function executeComputeProgram() {
    //gl.clear(gl.COLOR_BUFFER_BIT);
    gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);        
  }
  
  function deleteProgram(){
    gl.deleteProgram(this.program);
  }
  
  function useProgram(){
    gl.useProgram(this.program);
  }
  
  function setSpacing(x,y){
    gl.uniform2f(this.spacingUniform, x, y);
  }
  
  var setField1;
  var setField2;
  
  function initProgramsUtil(){
     setField1 = (useFloat ? setField1Float : setField1RGB565);
     setField2 = (useFloat ? setField2Float : setField2RGB565);
     setNormalField = setField2Float;
  }

  function setField1RGB565(field){
     setTexturesFromFieldRGB565(field, gl.TEXTURE0, gl.TEXTURE1, gl.TEXTURE2); 
  }

  function setField2RGB565(field){
     setTexturesFromFieldRGB565(field, gl.TEXTURE3, gl.TEXTURE4, gl.TEXTURE5); 
  }
  
  function setField1Float(field){
     setTextureFromFieldFloat(field, gl.TEXTURE0); 
  }
  
  function setField2Float(field){
     setTextureFromFieldFloat(field, gl.TEXTURE1);		 
  }
  
  function setField3Float(field){
     setTextureFromFieldFloat(field, gl.TEXTURE2);	 
  }
  
  function setField4Float(field){
     setTextureFromFieldFloat(field, gl.TEXTURE3);
  }
  
  function setField5Float(field){
     setTextureFromFieldFloat(field, gl.TEXTURE4);
  }
  
  function setTextureFromFieldFloat(field, location){
     gl.activeTexture(location);
     gl.bindTexture(gl.TEXTURE_2D, field.texture);
  }
  
  function setTexturesFromFieldRGB565(field, location1, location2, location3){
     gl.activeTexture(location1);
     gl.bindTexture(gl.TEXTURE_2D, field.texture1);
     gl.activeTexture(location2);
     gl.bindTexture(gl.TEXTURE_2D, field.texture2);
     gl.activeTexture(location3);
     gl.bindTexture(gl.TEXTURE_2D, field.texture3); 
  }
  
  function addAndMulExecOnFields(dstField, field1, field2){
    AddAndMulProgram.setField1(field1);
    AddAndMulProgram.setField2(field2);
    AddAndMulProgram.setDstField(dstField);
    AddAndMulProgram.execute();
  }
