// COMPUTE SHADERS

function getSrcComputeVS(){
    return srcHeadVS
  + "attribute vec2 pos;\n"
  + "varying vec2 texCoord;\n"
  + getDataWriteFields()
  + "void main(){\n"
  + " texCoord = pos;\n"
  + " gl_Position = vec4(pos * vec2(2.0) - vec2(1.0), 0.0, 1.0);\n"
  + "}\n";
}

function getSrcInitialPositionsFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 offset;\n"
  + "uniform vec2 scale;\n"
  + getDataWriteFields()
  + "void main(){\n"
  //+ "vec3 position = vec3(1.0,0.2,0.3);"
  //+ " vec3 position = vec3((texCoord + offset) * scale, 0.0);\n"
  + " vec3 position = vec3((texCoord + offset) * scale, (texCoord.x + offset.x) * 0.01 * (sin(texCoord.x * 30.0) + cos(texCoord.y * 30.0)));\n"
  + getDataWriter("position")
  + "}\n";
}

function getSrcSineWavePositionsFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform float time;\n"
  + getDataFieldGettersConstants()
  + getDataFields("position")
  + getDataWriteFields()
  + "void main(){\n"
  + " vec3 x1 = " + getDataFieldGetter("position", "texCoord") + ";\n"
  + " vec3 position = vec3(x1.xy, 0.1 * sin(20.0 * texCoord.x + time));\n"
  + getDataWriter("position")
  + "}\n";
}

function getSrcResetFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec3 value;\n"
  + getDataWriteFields() 
  + "void main(){\n"
  + getDataWriter("value")
  + "}\n"
}

function getSrcInteralParallelAccelerationFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n"
  + "uniform float neutralDistance;\n"
  + "uniform float factor;\n"
  + getDataFields("position")
  + getDataFields("acceleration")
  + getDataWriteFields() 
  + "vec3 getAccel(vec3 pos1, vec2 coor);\n"
    //
  + "void main(){\n"
  + " vec3 accel = " + getDataFieldGetter("acceleration", "texCoord") + ";\n"
  + " vec2 spacingX = vec2(spacing.x, 0.0);\n"
  + " vec2 spacingY = vec2(0.0, spacing.y);\n"
  + " vec3 pos = " + getDataFieldGetter("position", "texCoord") + ";\n"
      //
  + " vec2 coor = texCoord + spacingX;\n"
  + " if(coor.x < 1.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacingX;\n"
  + " if(coor.x > 0.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + spacingY;\n"
  + " if(coor.y < 1.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacingY;\n"
  + " if(coor.y > 0.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
  + getDataWriter("accel")
  + "}\n"
//
+ "vec3 getAccel(vec3 pos1, vec2 coor){\n"
+ " vec3 v = " + getDataFieldGetter("position", "coor") + " - pos1;\n"
+ " return vec3(factor * (neutralDistance - length(v))) * normalize(v);\n"
+ "}\n";
}

function getSrcInteralAccelerationFS(type){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n"
  + "uniform float l0;\n"
  + "uniform float kp;\n"
  + "uniform float kt;\n"
  + getDataFieldGettersConstants()
  + getDataFields("position")
  + getDataFields("acceleration")
  + getDataFields("velocity")
  + getDataWriteFields() 
  + "vec3 getAcceleration(vec3 x1, vec3 v1, vec2 coor);\n"
    //
  + "void main(){\n"
  + " vec3 a1 = " + getDataFieldGetter("acceleration", "texCoord") + ";\n"
  + " vec3 x1 = " + getDataFieldGetter("position", "texCoord") + ";\n"
  + " vec3 v1 = " + getDataFieldGetter("velocity", "texCoord") + ";\n"
      //
  + (type ? getInteralAccelerationParallelTests() : getInteralAccelerationDiagonalTests())
  + getDataWriter("a1")
  + "}\n"
//
+ "vec3 getAcceleration(vec3 x1, vec3 v1, vec2 coor){\n"
+ " vec3 x12 = x1 - " + getDataFieldGetter("position", "coor") + ";\n"
+ " vec3 n12 = normalize(x12);\n"
+ " vec3 vt = vec3(dot(" + getDataFieldGetter("velocity", "coor") + " - v1, n12)) * n12;\n"
//
+ " vec3 dl = n12 * vec3(l0) - x12;\n"
//
+ " return vec3(kt) * vt + vec3(kp) * dl;\n"
+ "}\n";
}

function getInteralAccelerationParallelTests(){
  return " vec2 spacingX = vec2(spacing.x, 0.0);\n"
  + " vec2 spacingY = vec2(0.0, spacing.y);\n"
  + " vec2 coor = texCoord + spacingX;\n"
  + " if(coor.x < 1.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacingX;\n"
  + " if(coor.x > 0.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + spacingY;\n"
  + " if(coor.y < 1.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacingY;\n"
  + " if(coor.y > 0.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n";
}

function getInteralAccelerationDiagonalTests(){
  return " vec2 coor = texCoord + spacing;\n"
  + " if(coor.x < 1.0 && coor.y < 1.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacing;\n"
  + " if(coor.x > 0.0 && coor.y > 0.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + vec2(spacing.x, -spacing.y);\n"
  + " if(coor.y > 0.0 && coor.x < 1.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + vec2(-spacing.x, spacing.y);\n"
  + " if(coor.y < 1.0 && coor.x > 0.0){\n"
  + "  a1 += getAcceleration(x1, v1, coor);\n"
  + " }\n";
}

function getSrcInteralDiagonalAccelerationFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n"
  + "uniform float neutralDistance;\n"
  + "uniform float factor;\n"
  //+ getDataFieldGettersConstants()
  + getDataFields("position")
  + getDataFields("acceleration")
  + getDataWriteFields() 
  + "vec3 getAccel(vec3 pos1, vec2 coor);\n"
    //
  + "void main(){\n"
  + " vec3 accel = " + getDataFieldGetter("acceleration", "texCoord") + ";\n"
  + " vec3 pos = " + getDataFieldGetter("position", "texCoord") + ";\n"
      //
  + " vec2 coor = texCoord + spacing;\n"
  + " if(coor.x < 1.0 && coor.y < 1.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord - spacing;\n"
  + " if(coor.x > 0.0 && coor.y > 0.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + vec2(spacing.x, -spacing.y);\n"
  + " if(coor.y > 0.0 && coor.x < 1.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
      //
  + " coor = texCoord + vec2(-spacing.x, spacing.y);\n"
  + " if(coor.y < 1.0 && coor.x > 0.0){\n"
  + "  accel += getAccel(pos, coor);\n"
  + " }\n"
  + getDataWriter("accel")
  + "}\n"
  //
  + "vec3 getAccel(vec3 pos1, vec2 coor){\n"
  + " vec3 v = " + getDataFieldGetter("position", "coor") + " - pos1;\n"
  + " return vec3(factor * (neutralDistance - length(v))) * normalize(v);\n"
  + "}\n";
}



function getSrcExteralAccelerationFS2(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n"
  + "uniform vec3 windVec;\n"
  + getDataFieldGettersConstants()
  + getDataFieldGetterFunctionDeclaration("position") 
  + getDataFields("position")
  + getDataWriteFields()
  + "void main(){\n"
  + " vec3 pos1 = getPosition(texCoord);\n"
  + " vec3 pos2 = getPosition(texCoord + vec2(spacing.x, 0.0));\n"
  + " vec3 pos3 = getPosition(texCoord + vec2(0.0, spacing.y));\n"
  + " vec3 pos4 = getPosition(texCoord + spacing);\n"
  + " vec3 a = vec3(pos2 - pos1);\n"
  + " vec3 b = vec3(pos3 - pos1);\n"
  + " vec3 normal1 = cross(a, b);\n"
  + " vec3 c = vec3(pos4 - pos2);\n"
  + " vec3 d = vec3(pos4 - pos3);\n"
  + " vec3 normal2 = cross(c, d);\n"
  + getDataWriter("dot(normal1, windVec) * normalize(normal1) + dot(normal2, windVec) * normalize(normal2)")
  + "}\n"
  + getDataFieldGetterFunction("position");
}

function getSrcExteralAccelerationFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n"
  + "uniform vec3 windVec;\n"
  + "uniform float cx;\n"
  + getDataFieldGettersConstants()
  + getDataFieldGetterFunctionDeclaration("position")
  + getDataFields("position")
  + getDataFieldGetterFunctionDeclaration("velocity") 
  + getDataFields("velocity")
  + getDataWriteFields()
  + "void main(){\n"
  + " vec3 x1 = getPosition(texCoord);\n"
  + " vec3 x2 = getPosition(texCoord + vec2(spacing.x, 0.0));\n"
  + " vec3 x3 = getPosition(texCoord + vec2(0.0, spacing.y));\n"
  + " vec3 x4 = getPosition(texCoord + spacing);\n"
  //
  + " vec3 v1 = getVelocity(texCoord);\n"
  + " vec3 v2 = getVelocity(texCoord + vec2(spacing.x, 0.0));\n"
  + " vec3 v3 = getVelocity(texCoord + vec2(0.0, spacing.y));\n"
  + " vec3 v4 = getVelocity(texCoord + spacing);\n"
  //
  + " vec3 vv1 = vec3(0.0 / 99.0) * (v1 + v2 + v3);\n"
  + " vec3 vv2 = vec3(0.0 / 99.0) * (v4 + v2 + v3);\n"
  //
  + " vec3 av1 = cross(x3 - x1, x2 - x1);\n"  // area vector 1 (x13, x12)
  + " vec3 av2 = cross(x4 - x2, x4 - x3);\n"  // area vector 2 (x24, x34)
  //
  + " vec3 vw1 = windVec - vv1;\n"
  + " vec3 vw2 = windVec - vv2;\n"
  //
  + " float spd1 = length(vw1);\n"
  + " float spd2 = length(vw2);\n"
  //
  + " vec3 wn = normalize(windVec);\n"
  + getDataWriter("vec3(cx) * (vec3(spd1 * spd1 * dot(av1, wn))* normalize(av1) + vec3(spd2 * spd2 * dot(av2, wn))* normalize(av2))")
  + "}\n"
  + getDataFieldGetterFunction("position")
  + getDataFieldGetterFunction("velocity");
}

function getSrcTextureOffsetFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 offset;\n"
  + getDataFieldGettersConstants()
  + getDataFields("acceleration")
  + getDataFields("windAcceleration")
  + getDataWriteFields()
  + "void main(){\n"
  + getDataWriter(getDataFieldGetter("acceleration", "texCoord") + " + " + getDataFieldGetter("windAcceleration", "texCoord + offset"))
  + "}\n";
}

function getSrcScaleFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec3 scale;\n"
  + getDataFieldGettersConstants()
  + getDataFields("position")
  + getDataWriteFields()
  + "void main(){\n"
  + " vec3 position = " + getDataFieldGetter("position", "texCoord") + ";\n"
  + getDataWriter("position", "(position * scale)")
  + "}\n"
}

function getSrcEulerIntegrationFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform float time;\n"
  + getDataFieldGettersConstants()
  + getDataFields("position")
  + getDataFields("velocity")
  + getDataWriteFields()
  //
  + "void main(){\n"
  + " vec3 position = " + getDataFieldGetter("position", "texCoord") + ";\n"
  + " vec3 velocity = " + getDataFieldGetter("velocity", "texCoord") + ";\n"
  + getDataWriter("position + velocity * vec3(time)")
  + "}\n"	
}

function getSrcRK4IntegrationFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform float time;\n"
  + getDataFields("src")
  + getDataFields("k1")
  + getDataFields("k2")
  + getDataFields("k3")
  + getDataFields("k4")
  + getDataWriteFields()
  //
  + "void main(){\n"
  + getDataWriter(getDataFieldGetter("src", "texCoord") + " + vec3(time * (1.0 / 6.0)) * (" + getDataFieldGetter("k1", "texCoord") + " + vec3(2.0) * (" +  getDataFieldGetter("k2", "texCoord") + " + " + getDataFieldGetter("k3", "texCoord") + ") + " +  getDataFieldGetter("k4", "texCoord") + ")")
  + "}\n"	
}

function getSrcAddAndMulFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform float scale;\n"
  + getDataFieldGettersConstants()
  + getDataFields("k1")
  + getDataFields("k2")
  + getDataWriteFields()
  //
  + "void main(){\n"
  + getDataWriter("vec3(scale) * (" + getDataFieldGetter("k1", "texCoord") + " + " + getDataFieldGetter("k2", "texCoord") + ")")
  + "}\n"	
}

function getSrcNormalFS(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform vec2 spacing;\n" // vzdálenost mezi sousedními pixely v textuře
  + getDataFieldGettersConstants()
  + getDataFields("position")
  //+ getDataWriteFields()
  + getDataFieldGetterFunctionDeclaration("position")
  + "void main(){\n"
  + " vec2 spacingX = vec2(spacing.x, 0.0);\n"
  + " vec2 spacingY = vec2(0.0, spacing.y);\n"
  + " vec3 dx = getPosition(texCoord + spacingX) - getPosition(texCoord - spacingX);\n"
  + " vec3 dy = getPosition(texCoord + spacingY) - getPosition(texCoord - spacingY);\n"
  + getNormalWriter("normalize(cross(dx, dy))")
  + "}\n"
  + getDataFieldGetterFunction("position");
}

// RENDERING SHADERS

function getSrcRenderingVS(){
    return srcHeadVS	  
  + "uniform mat4 transformationMatrix;\n"
  + getDataFieldGettersConstants()
  + renderingVSFields() 
  + "varying vec3 normal;\n"
  + "void main(){\n"
  + " vec4 pos = vec4(" + renderingVSPositionGetter() + ", 1.0);\n"
  //+ " pos.z = 0.0;\n"
  + " gl_Position = transformationMatrix * pos;\n"
  + " normal = " + renderingVSNormalGetter() + ";\n"
  + "}\n";
}

function getSrcRenderingVBOVS(){
    "attribute vec3 positionIn;\n"
  + "attribute vec3 normalIn;\n"
  + "uniform mat4 transformationMatrix;\n"
  + "uniform mat3 normalMatrix;\n"
  + "varying vec3 normal;\n"
  + "void main(){\n"
  + " vec4 pos = vec4(positionIn, 1.0);\n"
  + " gl_Position = transformationMatrix * pos;\n"
  + " normal = normalMatrix * normalIn;\n"
  + "}\n";
}

function getSrcRenderingFS(){
    return srcHeadFS
  + "varying vec3 normal;\n"
  + "uniform vec3 lightVec;\n"
  + "uniform vec3 color;\n"
  + "void main(){\n"
  //+ " gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0);\n"
  + " gl_FragColor = vec4(vec3(abs(dot(normalize(normal), lightVec))) * color, 1.0);\n"
  + "}\n";
}

function getSrc(){
    return srcHeadFS
  + "varying vec2 texCoord;\n"
  + "uniform sampler2D texture;\n"
  + "void main(){\n"
  + " gl_FragColor = vec4(texture2D(texture, texCoord).xyz, 1.0);\n"
  + "}\n";
}
