var srcHeadVS =
"#version 100\n";

var srcHeadFS =
"#version 100\n"
+ "precision highp float;\n";

// DATA FIELD GETTERS

function getDataFields(name){
   if(useFloat){
       return "uniform sampler2D " + name + "Field;\n";
   } else {
       return "uniform sampler2D " + name + "Field1;\n"
         + "uniform sampler2D " + name + "Field2;\n"
         + "uniform sampler2D " + name + "Field3;\n"
         + "uniform vec3  " + name + "Offset;\n"
         + "uniform vec3  " + name + "Scale;\n";
   }
}

function getDataFieldGetter(name, coor){
   if(useFloat){
       return getDataFieldGetterSingleTexture(name, coor);
   } else {
       return getDataFieldGetter565Textures(name, coor);
   }
}

function getDataFieldGetterSingleTexture(name, coor){
     return "texture2D(" + name + "Field, " + coor + ").rgb"
}

//
// 
//
//
//

//   | r | g | b |
// x | 1 | 2 | 3 |
// y | 3 | 1 | 2 |
// z | 2 | 3 | 1 |
//
// float c1 = (32.0 * 64.0 * 32.0) / 65535.0;
// float c2 = (32.0 * 64.0) / 65535.0;
// float c3 = 32.0 / 65535.0;
//
// read   (T1.rgb * vec3(c1, c2, c3) + T2.rgb * vec3(c2, c3, c1) + T3.rgb * vec3(c3, c1, c2) + offset) * scale
//
// scaledDstVal = vec3(X.XXXXXXXX...b)
//
// write  fract(floor(scaledDstVal.xyz * w1) * w2)
// cw1 = 31.0;
// cw2 = 31.0 * 32.0;
// cw3 = 31.0 * 32.0 * 32.0;
//
//  1: w1 = vec3(); w2 = vec3();
//  2: w1 = vec3(); w2 = vec3();
//  3: w2 = vec3(); w2 = vec3();
//
// read  vec(T1.r, T2.r, T3.r) * vec3(c) + vec(T1.g, T2.g, T3.g) * vec3(c) + vec(T1.b, T2.b, T3.b) * vec3(c) // 9 * vector. , 3 * mad
// write DstColor[] * c // 1 * mul

function getDataFieldGettersConstants(){
	return (useFloat ? "" : getDataFieldGettersConstants565Textures());
}

function getDataFieldGettersConstants565Textures(){
	return "const float c_component1 = (31.0 * 64.0 * 32.0) / 65535.0;\n"
	+ "const float c_component2 = (63.0 * 32.0) / 65535.0;\n"
	+ "const float c_component3 = 31.0 / 65535.0;\n"
	+ "const vec3 c_Field = vec3(c_component1, c_component2, c_component3);\n"
	//+ "const vec3 c_Field2 = vec3(c_component2, c_component3, c_component1);\n"
	//+ "const vec3 c_Field3 = vec3(c_component3, c_component1, c_component2);\n";
}

function getDataFieldGetter565Textures(name, coor){
	//
        return "((texture2D(" + name + "Field1, " + coor + ").xyz * c_Field.xyz + texture2D(" + name + "Field2, " + coor + ").yzx * c_Field.yzx + texture2D(" + name + "Field3, " + coor + ").zxy * c_Field.zxy + " + name + "Offset) * " + name + "Scale)";
}

function getDataFieldGetterFunction(name){
    return "vec3 get" + getNameFirstCapital(name) + "(vec2 texCoor){\n"
  + " return " + getDataFieldGetter(name, "texCoor") + ";\n"
  + "}\n"
}

function getDataFieldGetterFunctionDeclaration(name){
    return "vec3 get" + getNameFirstCapital(name) + "(vec2 texCoor);\n"; 
}

function getNameFirstCapital(name){
    return name.substring(0, 1).toUpperCase() + name.substring(1);
}

// DATA WRITERS

function getDataWriteFields(){
   if(useFloat){
       return "";
   } else {
       return "uniform vec3 dataOffset;\n"
     + "uniform vec3 dataScale;\n"
     + "uniform int dataComponentsIndex;\n"; // 1 2 3  |  2 3 1  |  3 1 2
     //+ "constant c_data1 = ;\n"
     //+ "constant c_data2 = ;\n"
     //+ "constant c_data3 = ;\n";
   }
}

function getDataWriter(data){


//   | r | g | b |
// x | 1 | 2 | 3 |
// y | 2 | 3 | 1 |
// z | 3 | 1 | 2 |
   if(useFloat){
       return " gl_FragData[0] = vec4(" + data + ", 0.0);\n";
   } else {	   
       return " vec3 dataValueTmp = " + data + ";\n"
       + " vec3 dataValue = (dataComponentsIndex == 0) ? dataValueTmp.xyz : ((dataComponentsIndex == 1) ? dataValueTmp.zxy : dataValueTmp.yzx);\n" 
       + " gl_FragData[0] = vec4(floor(fract(min((dataValue * dataScale + dataOffset) * vec3(65535.0), vec3(65535.0)) * vec3(1.0 / 65536.0, 1.0 /  2048.0, 1.0 / 32.0)) * vec3(32.0, 64.0, 32.0)) * vec3(1.0 / 31.0, 1.0 / 63.0, 1.0 / 31.0), 0.0);\n";
       //return " gl_FragData[0] = vec4(floor(fract((" + data + ") * dataScale + dataOffset) * vec3(32.0, 64.0, 32.0)), 0.0);\n";
   }
}

function getNormalWriter(data){
   if(useFloat){
       return " gl_FragData[0] = vec4(" + data + ", 0.0);\n";
   } else {	   
       return " gl_FragData[0] = vec4((" + data + ") * 0.5 + 0.5, 0.0);\n"; 
   }
}

// RENDERING VS

function renderingVSFields(){
  if(vertexTexture){
    return "attribute vec2 inTexCoor;\n"
    +"uniform sampler2D normalField;\n"
    + getDataFields("position")
  } else {
    return "attribute vec3 inPosition;"
    + "attribute vec2 inNormal;"
  }
}

function renderingVSPositionGetter(){
  if(vertexTexture){
    return getDataFieldGetter("position", "inTexCoor");	  	  
  } else {
    return "inPosition";
  }
}

function renderingVSNormalGetter(){
  if(vertexTexture){
    if(useFloat){
      return getDataFieldGetterSingleTexture("normal", "inTexCoor");
    } else {
      return getDataFieldGetterSingleTexture("normal", "inTexCoor"); + " * 2.0 - 1.0";	
    }
  } else {
    return "inNormal";
  }
}
